package KorAP::ScriptLoad;
use KorAP::VirtualCorpus::Group;
use KorAP::VirtualCorpus::Or;
use KorAP::VirtualCorpus::GroupRef;
use File::Spec::Functions qw!catfile!;
use Mojo::Util qw'encode';
use strict;
use warnings;


# Constructor
sub new {
  my $class = shift;
  my %self = @_;

  $self{output} //= '.';

  my $file = $self{file};

  if (ref $file && ref $file eq 'GLOB') {
    $self{file} = '';
    $self{fh} = $file;
    return bless \%self, $class;
  };

  # Open def file
  if (!open($self{fh}, '<' . $file)) {
    warn $file . " can't be opened";
    exit(0);
  };

  return bless \%self, $class;
};


# Parse script file
sub parse {
  my $self = shift;

  while (!eof($self->{fh})) {
    my $line = readline($self->{fh});

    next if $line =~ /^\#/;

    if ($line =~ /LOAD/) {
      my ($name, $desc, @vcs) = _parse_load($line);
      unless ($name) {
        warn
      };
      $self->load_to_vc($name, $desc, @vcs);
    };
  };

  close($self->{fh});
};

sub load_to_vc {
  my $self = shift;
  my ($name, $desc, @list) = @_;

  print "Convert '", $name, "' from LOAD()\n";

  my $vc = from_load($name, $desc, @list);

  # Output
  my $out = catfile($self->{output}, $name . '.jsonld');
  if (open(my $koral, '>' . encode('utf-8', $out))) {
    binmode($koral);
    print $koral encode('utf-8', $vc->to_string);
    close($koral);
    return;
  };

  warn 'Unable to write file ' . $out;
};

sub _parse_load {
  my $line = shift;
  chomp($line);
  $line =~ s/^\s*echo\s+-e\s+//;
  $line =~ s/\s+>>\s*\$.+$//;
  $line =~ s/^\s*"(.+?)"\s*$/$1/;
  $line =~ s/\\t/ /g;

  $line =~ /^(.+?) - (.+?)[\s\t]+(?:(?:_[^_]+_)+[\s\t]+)?LOAD\(([^\)]+?)\).*$/;
  my ($name, $desc, $list) = ($1, $2, $3);
  $list =~ s/^"(.+?)"$/$1/;
  $list =~ s/^'(.+?)'$/$1/;
  return $name, $desc, split(/\s+/,$list);
};


sub from_load {
  my ($name, $desc, @vcs) = @_;

  my @vc_objs = ();
  foreach (@vcs) {

    if (index($_,'-') == 0) {
      my $minus = substr($_,1);

      my $neg_vc = KorAP::VirtualCorpus::GroupRef->new($minus);
      $neg_vc->match('ne');

      my $base_vc = KorAP::VirtualCorpus::Or->new(@vc_objs);
      (@vc_objs) = KorAP::VirtualCorpus::And->new($base_vc, $neg_vc);
    }

    else {
      push @vc_objs, KorAP::VirtualCorpus::GroupRef->new($_);
    };
  };

  if (@vc_objs > 1) {
    return KorAP::VirtualCorpus::Or->new(@vc_objs);
  };
  return $vc_objs[0];
};

1;
